@@ -40,8 +40,11 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtkalignment.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkdnd.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
 #include <gtk/gtkselection.h>
@@ -70,6 +73,37 @@
  */
 #define MAXIMUM_WM_REPARENTING_DEPTH 4
 
+/**
+ * eel_gtk_button_new_with_stock_icon:
+ * 
+ * @label: the button label, which may include a mnemonic accelerator.
+ * @stock_id: the stock icon to use.
+ *
+ * Creates a GtkButton with a mnemonic label and a stock icon.
+ *
+ * Return value: a new GtkButton widget.
+ **/
+GtkWidget*
+eel_gtk_button_new_with_stock_icon (const gchar *label, const gchar *stock_id)
+{
+	GtkWidget *button, *l, *image, *hbox, *align;
+
+	/* This is mainly copied from gtk_button_construct_child(). */
+	button = gtk_button_new ();
+	l = gtk_label_new_with_mnemonic (label);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (l), GTK_WIDGET (button));
+	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+	hbox = gtk_hbox_new (FALSE, 2);
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (hbox), l, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (button), align);
+	gtk_container_add (GTK_CONTAINER (align), hbox);
+	gtk_widget_show_all (align);
+
+	return button;
+}
+
 /**
  * eel_gtk_button_set_padding
  * 
@@ -330,7 +364,9 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
 					  guint width,
 					  guint height)
 {
+	GdkScreen *screen;
 	int real_left, real_top;
+	int screen_width, screen_height;
 
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	g_return_if_fail (!(geometry_flags & EEL_GDK_WIDTH_VALUE) || width > 0);
@@ -346,6 +382,10 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
 		real_left = left;
 		real_top = top;
 
+		screen = gtk_window_get_screen (window);
+		screen_width  = gdk_screen_get_width  (screen);
+		screen_height = gdk_screen_get_height (screen);
+
 		/* This is sub-optimal. GDK doesn't allow us to set win_gravity
 		 * to South/East types, which should be done if using negative
 		 * positions (so that the right or bottom edge of the window
@@ -353,10 +393,10 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
 		 * However it does seem to be consistent with other GNOME apps.
 		 */
 		if (geometry_flags & EEL_GDK_X_NEGATIVE) {
-			real_left = gdk_screen_width () - real_left;
+			real_left = screen_width - real_left;
 		}
 		if (geometry_flags & EEL_GDK_Y_NEGATIVE) {
-			real_top = gdk_screen_height () - real_top;
+			real_top = screen_height - real_top;
 		}
 
 		sanity_check_window_position (&real_left, &real_top);
@@ -379,17 +419,19 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
  * some sanity-checking on the passed-in values.
  * 
  * @window: A non-visible GtkWindow
- * @geometry_string: A string suitable for use with gnome_parse_geometry
+ * @geometry_string: A string suitable for use with eel_gdk_parse_geometry
  * @minimum_width: If the width from the string is smaller than this,
  * use this for the width.
  * @minimum_height: If the height from the string is smaller than this,
  * use this for the height.
+ * @ignore_position: If true position data from string will be ignored.
  */
 void
 eel_gtk_window_set_initial_geometry_from_string (GtkWindow *window, 
 						 const char *geometry_string,
 						 guint minimum_width,
-						 guint minimum_height)
+						 guint minimum_height,
+						 gboolean ignore_position)
 {
 	int left, top;
 	guint width, height;
@@ -415,47 +457,15 @@ eel_gtk_window_set_initial_geometry_from_string (GtkWindow *window,
 	if (geometry_flags & EEL_GDK_HEIGHT_VALUE) {
 		height = MAX (height, minimum_height);
 	}
+	
+	/* Ignore saved window position if requested. */
+	if (ignore_position) {
+		geometry_flags &= ~(EEL_GDK_X_VALUE | EEL_GDK_Y_VALUE);
+	}
 
 	eel_gtk_window_set_initial_geometry (window, geometry_flags, left, top, width, height);
 }
 
-/**
- * eel_gtk_signal_connect_free_data:
- * 
- * Function to displace the popup menu some, otherwise the first item
- * gets selected right away.
- * This function gets called by gtk_menu_popup ().
- *
- * @menu: the popup menu.
- * @x: x coord where gtk want to place the menu
- * @y: y coord where gtk want to place the menu
- * @user_data: something
- **/
-static void
-eel_popup_menu_position_func (GtkMenu  *menu,
-			      int      *x,
-			      int      *y,
-			      gboolean *push_in,
-			      gpointer  user_data)
-{
-	GdkPoint *offset;
-	GtkRequisition requisition;
-
-	g_assert (x != NULL);
-	g_assert (y != NULL);
-
-	offset = (GdkPoint *) user_data;
-
-	g_assert (offset != NULL);
-
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
-	  
-	*x = CLAMP (*x + (int) offset->x, 0, MAX (0, gdk_screen_width () - requisition.width));
-	*y = CLAMP (*y + (int) offset->y, 0, MAX (0, gdk_screen_height () - requisition.height));
-
-	*push_in = FALSE;
-}
-
 /**
  * eel_truncate_text_for_menu_item:
  * 
@@ -515,7 +525,7 @@ eel_pop_up_context_menu (GtkMenu	     *menu,
 	gtk_menu_popup (menu,					/* menu */
 			NULL,					/* parent_menu_shell */
 			NULL,					/* parent_menu_item */
-			eel_popup_menu_position_func,	/* func */
+			NULL,
 			&offset,			        /* data */
 			button,					/* button */
 			event ? event->time : GDK_CURRENT_TIME); /* activate_time */
@@ -1226,6 +1236,56 @@ eel_gtk_tree_view_cell_is_completely_visible (GtkTreeView          *tree_view,
 	return eel_gdk_rectangle_contains_rectangle (visible_rect, cell_rect);
 }
 
+static gboolean 
+tree_view_button_press_callback (GtkWidget *tree_view,
+				 GdkEventButton *event,
+				 gpointer data)
+{
+	GtkTreePath *path;
+	GtkTreeViewColumn *column;
+
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view),
+						   event->x, event->y,
+						   &path,
+						   &column,
+						   NULL, 
+						   NULL)) {
+			gtk_tree_view_row_activated
+				(GTK_TREE_VIEW (tree_view), path, column);
+		}
+	}
+
+	return FALSE;
+}
+
+void
+eel_gtk_tree_view_set_activate_on_single_click (GtkTreeView *tree_view,
+						gboolean should_activate)
+{
+	guint button_press_id;
+
+	button_press_id = GPOINTER_TO_UINT 
+		(g_object_get_data (G_OBJECT (tree_view), 
+				    "eel-tree-view-activate"));
+
+	if (button_press_id && !should_activate) {
+		g_signal_handler_disconnect (tree_view, button_press_id);
+		g_object_set_data (G_OBJECT (tree_view), 
+				   "eel-tree-view-activate", 
+				   0);
+	} else if (!button_press_id && should_activate) {
+		button_press_id = g_signal_connect 
+			(tree_view,
+			 "button_press_event",
+			 G_CALLBACK  (tree_view_button_press_callback),
+			 NULL);
+		g_object_set_data (G_OBJECT (tree_view), 
+				   "eel-tree-view-activate", 
+				   GUINT_TO_POINTER (button_press_id));
+	}
+}
+
 gboolean
 eel_gtk_viewport_get_visible_rect (GtkViewport  *viewport, 
 				   GdkRectangle *rect)
